/*----------------------------------------------------------------------------
 * File:  Tracking_ports.c
 *
 * UML Component Port Messages
 * Component Name:                       Tracking
 * Domain Name (if modeled internally):  Tracking
 *
 * (C) Copyright 1998-2009 Mentor Graphics Corporation.  All rights reserved.
 *--------------------------------------------------------------------------*/

#include "sys_types.h"
#include "Tracking_ports.h"
#include "HeartRateMonitor_ports.h"
#include "Location_ports.h"
#include "UI_ports.h"
#include "Tracking_classes.h"

/*
 * Interface:                 UI
 * Required Port:             UI
 * Client to Server Message:  setTime
 */
void
Tracking_UI_setTime( i_t p_time )
{
UI_UI_setTime(   p_time );
}

/*
 * Interface:                 UI
 * Required Port:             UI
 * Client to Server Message:  setData
 */
void
Tracking_UI_setData( s2_t p_unit, r4_t p_value )
{
UI_UI_setData(   p_unit,  p_value );
}

/*
 * Interface:                 LocationUtil
 * Required Port:             UTIL
 * Client to Server Message:  getDistance
 */
r4_t
Tracking_UTIL_getDistance( sys_sdt_Location p_fromLocation, sys_sdt_Location p_toLocation )
{
return Location_UTIL_getDistance(   p_fromLocation,  p_toLocation );
}

/*
 * Interface:                 LocationProvider
 * Required Port:             LOC
 * Server to Client Message:  locationUpdate
 */
void
Tracking_LOC_locationUpdate( sys_sdt_Location p_location )
{
  Tracking_TrackLog * trackLog;
  /* LOG::LogInfo( message:'location updated: ' ) */
  ROX_BPAL_STMT_TRACE( 1, "LOG::LogInfo( message:'location updated: ' )" )
  LOG_LogInfo( "location updated: " );
  /* LOG::LogReal( message:'longitude', r:PARAM.location.longitude ) */
  ROX_BPAL_STMT_TRACE( 1, "LOG::LogReal( message:'longitude', r:PARAM.location.longitude )" )
  LOG_LogReal( "longitude", p_location.longitude );
  /* LOG::LogReal( message:'latitude', r:PARAM.location.latitude ) */
  ROX_BPAL_STMT_TRACE( 1, "LOG::LogReal( message:'latitude', r:PARAM.location.latitude )" )
  LOG_LogReal( "latitude", p_location.latitude );
  /* LOG::LogReal( message:'speed', r:PARAM.location.speed ) */
  ROX_BPAL_STMT_TRACE( 1, "LOG::LogReal( message:'speed', r:PARAM.location.speed )" )
  LOG_LogReal( "speed", p_location.speed );
  /* SELECT any trackLog FROM INSTANCES OF TrackLog */
  ROX_BPAL_STMT_TRACE( 1, "SELECT any trackLog FROM INSTANCES OF TrackLog" )
  trackLog = (Tracking_TrackLog *) Escher_SetGetAny( &pG_Tracking_TrackLog_extent.active );
  /* IF ( not empty trackLog ) */
  ROX_BPAL_STMT_TRACE( 1, "IF ( not empty trackLog )" )
  if ( !( 0 == trackLog ) ) {
    /* trackLog.addTrackPoint( location:PARAM.location ) */
    ROX_BPAL_STMT_TRACE( 2, "trackLog.addTrackPoint( location:PARAM.location )" )
    Tracking_TrackLog_op_addTrackPoint( trackLog,  p_location );
  }
}

/*
 * Interface:                 LocationProvider
 * Required Port:             LOC
 * Client to Server Message:  registerListener
 */
void
Tracking_LOC_registerListener( void )
{
Location_LOC_registerListener();
}

/*
 * Interface:                 LocationProvider
 * Required Port:             LOC
 * Client to Server Message:  unregisterListener
 */
void
Tracking_LOC_unregisterListener( void )
{
Location_LOC_unregisterListener();
}

/*
 * Interface:                 UI
 * Required Port:             UI
 * Server to Client Message:  lapResetPressed
 */
void
Tracking_UI_lapResetPressed( void )
{
  Tracking_WorkoutTimer * workoutTimer;
  /* SELECT any workoutTimer FROM INSTANCES OF WorkoutTimer */
  ROX_BPAL_STMT_TRACE( 1, "SELECT any workoutTimer FROM INSTANCES OF WorkoutTimer" )
  workoutTimer = (Tracking_WorkoutTimer *) Escher_SetGetAny( &pG_Tracking_WorkoutTimer_extent.active );
  /* IF ( not empty workoutTimer ) */
  ROX_BPAL_STMT_TRACE( 1, "IF ( not empty workoutTimer )" )
  if ( !( 0 == workoutTimer ) ) {
    /* GENERATE WorkoutTimer2:lapResetPressed() TO workoutTimer */
    ROX_BPAL_STMT_TRACE( 2, "GENERATE WorkoutTimer2:lapResetPressed() TO workoutTimer" )
    {
  Escher_xtUMLEvent_t * e = Escher_NewxtUMLEvent( workoutTimer, &Tracking_WorkoutTimerevent2c );
  Escher_SendEvent( e );
  }
  }
}

/*
 * Interface:                 UI
 * Required Port:             UI
 * Server to Client Message:  startStopPressed
 */
void
Tracking_UI_startStopPressed( void )
{
  Tracking_WorkoutTimer * workoutTimer;
  /* SELECT any workoutTimer FROM INSTANCES OF WorkoutTimer */
  ROX_BPAL_STMT_TRACE( 1, "SELECT any workoutTimer FROM INSTANCES OF WorkoutTimer" )
  workoutTimer = (Tracking_WorkoutTimer *) Escher_SetGetAny( &pG_Tracking_WorkoutTimer_extent.active );
  /* IF ( empty workoutTimer ) */
  ROX_BPAL_STMT_TRACE( 1, "IF ( empty workoutTimer )" )
  if ( ( 0 == workoutTimer ) ) {
    /* CREATE OBJECT INSTANCE workoutTimer OF WorkoutTimer */
    ROX_BPAL_STMT_TRACE( 2, "CREATE OBJECT INSTANCE workoutTimer OF WorkoutTimer" )
    workoutTimer = (Tracking_WorkoutTimer *) Escher_CreateInstance( Tracking_DOMAIN_ID, Tracking_WorkoutTimer_CLASS_NUMBER );
    /* ASSIGN workoutTimer.time = 0 */
    ROX_BPAL_STMT_TRACE( 2, "ASSIGN workoutTimer.time = 0" )
    workoutTimer->time = 0;
  }
  /* GENERATE WorkoutTimer1:startStopPressed() TO workoutTimer */
  ROX_BPAL_STMT_TRACE( 1, "GENERATE WorkoutTimer1:startStopPressed() TO workoutTimer" )
  {
  Escher_xtUMLEvent_t * e = Escher_NewxtUMLEvent( workoutTimer, &Tracking_WorkoutTimerevent1c );
  Escher_SendEvent( e );
  }
}

/*
 * Interface:                 UI
 * Required Port:             UI
 * Server to Client Message:  lightPressed
 */
void
Tracking_UI_lightPressed( void )
{
}

/*
 * Interface:                 UI
 * Required Port:             UI
 * Server to Client Message:  modePressed
 */
void
Tracking_UI_modePressed( void )
{
{
  Escher_xtUMLEvent_t * e = Escher_NewxtUMLEvent( (void *) 0, &Tracking_Display_CBevent1c );
  Escher_SendEvent( e );
  }

}

/*
 * Interface:                 UI
 * Required Port:             UI
 * Server to Client Message:  setTargetPressed
 */
void
Tracking_UI_setTargetPressed( void )
{
}

/*
 * Interface:                 HeartRateProvider
 * Required Port:             HR
 * Server to Client Message:  heartRateChanged
 */
void
Tracking_HR_heartRateChanged( r4_t p_heartRate )
{
  Tracking_TrackLog * trackLog;
  /* SELECT any trackLog FROM INSTANCES OF TrackLog */
  ROX_BPAL_STMT_TRACE( 1, "SELECT any trackLog FROM INSTANCES OF TrackLog" )
  trackLog = (Tracking_TrackLog *) Escher_SetGetAny( &pG_Tracking_TrackLog_extent.active );
  /* IF ( not empty trackLog ) */
  ROX_BPAL_STMT_TRACE( 1, "IF ( not empty trackLog )" )
  if ( !( 0 == trackLog ) ) {
    /* trackLog.addHeartRateSample( heartRate:PARAM.heartRate ) */
    ROX_BPAL_STMT_TRACE( 2, "trackLog.addHeartRateSample( heartRate:PARAM.heartRate )" )
    Tracking_TrackLog_op_addHeartRateSample( trackLog,  p_heartRate );
  }
}

/*
 * Interface:                 HeartRateProvider
 * Required Port:             HR
 * Client to Server Message:  registerListener
 */
void
Tracking_HR_registerListener( void )
{
HeartRateMonitor_HR_registerListener();
}

/*
 * Interface:                 HeartRateProvider
 * Required Port:             HR
 * Client to Server Message:  unregisterListener
 */
void
Tracking_HR_unregisterListener( void )
{
HeartRateMonitor_HR_unregisterListener();
}

