/*----------------------------------------------------------------------------
 * File:  sys_sets.h
 *
 * (C) Copyright 1998-2009 Mentor Graphics Corporation.  All rights reserved.
 *--------------------------------------------------------------------------*/

#ifndef SYS_SETS_H
#define SYS_SETS_H

#ifdef	__cplusplus
extern "C" {
#endif

/*
 * This is the most simple and basic container class and
 * represents elements in the set (nodes in the list).
 */
typedef struct Escher_SetElement_s Escher_SetElement_s;
struct Escher_SetElement_s {
  Escher_SetElement_s * next;
  void * object;
};

/*
 * This is the set class and may contain attributes that apply
 * to the set as a whole.
 */
typedef struct {
  Escher_SetElement_s * head;
} Escher_ObjectSet_s;

/*
 * Notice the symmetry between the object containoid, the
 * head of set structure and the set node containoid.
 */
typedef struct {
  Escher_SetElement_s * next;
} Escher_Object_s;

/*
 * Here is the shape of the general class extent.  This array contains list
 * heads for both active (animate) and inactive (inanimate) instances.
 */
typedef struct {
  Escher_ObjectSet_s active, inactive;
  Escher_SetElement_s* container;
  Escher_iHandle_t pool;
  Escher_ClassSize_t size;
  Escher_StateNumber_t initial_state;
  Escher_InstanceIndex_t population;
} Escher_Extent_t;

extern void Escher_SetFactoryInit( void );
extern void Escher_CopySet( Escher_ObjectSet_s *,
                Escher_ObjectSet_s * const );
extern void Escher_ClearSet( Escher_ObjectSet_s * );
extern void Escher_SetInsertElement( Escher_ObjectSet_s *,
                         void * const );
extern Escher_SetElement_s *
Escher_SetInsertBlock( Escher_SetElement_s *,
                       const u1_t *,
                       const u2_t,
                       u2_t );
#define Escher_SetRemoveInstance( pextent, instance, slot, container, pool ) \
  slot = Escher_SetRemoveNode( pextent, instance )
#define Escher_SetInsertInstance( pextent, node ) \
  node->next = (pextent)->head; \
  (pextent)->head = node


extern Escher_SetElement_s *
Escher_SetRemoveNode( Escher_ObjectSet_s *,
                      const void * const );
extern void Escher_SetRemoveElement( Escher_ObjectSet_s *,
                         const void * const );
extern const void * Escher_SetContains( const Escher_ObjectSet_s * const,
                    const void * const );
extern u2_t Escher_SetCardinality( const Escher_ObjectSet_s * const );
extern bool Escher_SetEquality( Escher_ObjectSet_s * const,
                    Escher_ObjectSet_s * const );
#define Escher_InitSet( S ) (S)->head = 0
#define Escher_SetGetAny( S ) ( ((S)->head != 0) ? (S)->head->object : 0 )
#define Escher_SetIsEmpty( S ) ( (S)->head == 0 )

/*
 * Collection Iteration
 * The iterator uses a cursor external to the set (or extent).
 * This cursor maintains a flavor of current element.
 */
typedef struct Iterator_s Iterator_s;
struct Iterator_s {
  Escher_SetElement_s * cursor;
};
#define Escher_IteratorReset( I, S ) ( ((I)->cursor = (S)->head) )
extern void * Escher_IteratorNext( Iterator_s * const );

/* We could easily replace this function declaration with a macro
   that invoked the compiler (C library) supplied strlen.  */
extern u2_t Escher_strlen( const c_t * );

/* We could easily replace this function declaration with a macro
   that invoked the compiler (C library) supplied memset.  */
extern void Escher_memset( void * const, const u1_t, u2_t );

/* We could easily replace this function declaration with a macro
   that invoked the compiler (C library) supplied memmove.  */
extern void Escher_memmove( void * const, const void * const, u2_t );
extern void Escher_strcpy( c_t *, const c_t * );
extern c_t * Escher_stradd( const c_t *, const c_t * );
/* We could easily replace this function declaration with a macro
   that invoked the compiler (C library) supplied strcmp.  */
extern c_t Escher_strcmp( const c_t *, const c_t * );

#ifdef	__cplusplus
}
#endif

#endif  /* SYS_SETS_H */
