/*----------------------------------------------------------------------------
 * File:  Tracking_TrackLog_class.h
 *
 * Class:       TrackLog  (TrackLog)
 * Subsystem:   Tracking
 * Domain:      Tracking
 *
 * (C) Copyright 1998-2009 Mentor Graphics Corporation.  All rights reserved.
 *--------------------------------------------------------------------------*/

#ifndef TRACKING_TRACKLOG_CLASS_H
#define TRACKING_TRACKLOG_CLASS_H

#ifdef	__cplusplus
extern "C" {
#endif

/*
 * Structural representation of application analysis class:
 *   TrackLog  (TrackLog)
 */
struct Tracking_TrackLog {

  /* application analysis class attributes */
  r4_t distance;  /* - distance */
  r4_t currentSpeed;  /* - currentSpeed */
  r4_t currentPace;  /* - currentPace (M) */
  r4_t currentHeartRate;  /* - currentHeartRate */
  Escher_TimeStamp_t startTime;  /* - startTime */
  sys_sdt_Location lastKnownLocation;  /* - lastKnownLocation */
  bool hasLocation;  /* - hasLocation */

  /* relationship storage */
  Tracking_TrackPoint * mc_TrackPoint_R1;
  Tracking_TrackPoint * mc_TrackPoint_R3;
  Tracking_WorkoutTimer * mc_WorkoutTimer_R4;
  Escher_ObjectSet_s mc_LapMarker_R5;
  Escher_ObjectSet_s mc_HeartRateSample_R6;
};

#define Tracking_TrackLog_MAX_EXTENT_SIZE 200
extern Escher_Extent_t pG_Tracking_TrackLog_extent;
extern void Tracking_TrackLog_op_addTrackPoint( Tracking_TrackLog *, sys_sdt_Location );
extern void Tracking_TrackLog_op_clearTrackPoints( Tracking_TrackLog * );
extern void Tracking_TrackLog_op_addLapMarker( Tracking_TrackLog * );
extern void Tracking_TrackLog_op_clearLapMarkers( Tracking_TrackLog * );
extern void Tracking_TrackLog_op_addHeartRateSample( Tracking_TrackLog *, r4_t );
extern void Tracking_TrackLog_op_clearHeartRateSamples( Tracking_TrackLog * );
extern void Tracking_TrackLog_op_init( Tracking_TrackLog * );

extern void Tracking_TrackLog_R1_Link( Tracking_TrackPoint *, Tracking_TrackLog * );
extern void Tracking_TrackLog_R1_Unlink( Tracking_TrackPoint *, Tracking_TrackLog * );
#define Tracking_LapMarker_R5_From_TrackLog( TrackLog ) ( &((TrackLog)->mc_LapMarker_R5) )
#define Tracking_HeartRateSample_R6_From_TrackLog( TrackLog ) ( &((TrackLog)->mc_HeartRateSample_R6) )

r4_t Tracking_TrackLog_MDA_currentPace( Tracking_TrackLog * );

#ifdef	__cplusplus
}
#endif

#endif  /* TRACKING_TRACKLOG_CLASS_H */


